@@ -14,23 +14,23 @@
 #include "identityfolderrequester.h"
 #include "identityinvalidfolder.h"
 
-#include <MessageComposer/MessageComposerSettings>
+
 #include <QGpgME/Job>
 #include <QGpgME/Protocol>
 
 #include <KIdentityManagement/IdentityManager>
 
 // other KMail headers:
+#include "settings/kmailsettings.h"
 #include "xfaceconfigurator.h"
 #include <KEditListWidget>
 #include <MailCommon/FolderRequester>
-#include "kmkernel.h"
-#include "settings/kmailsettings.h"
 
 #include <MailCommon/MailKernel>
 
 #include "job/addressvalidationjob.h"
 #include <MessageComposer/Kleo_Util>
+#include <MessageComposer/MessageComposerSettings>
 #include <MessageCore/StringUtil>
 #include <Sonnet/DictionaryComboBox>
 #include <TemplateParser/TemplatesConfiguration>
@@ -39,8 +39,8 @@
 #include <KIdentityManagement/Identity>
 #include <KIdentityManagement/SignatureConfigurator>
 
-#include <PimCommon/AutoCorrectionLanguage>
 #include <PimCommon/PimUtil>
+#include <PimCommonAutoCorrection/AutoCorrectionLanguage>
 
 #include <Libkdepim/LineEditCatchReturnKey>
 #include <PimCommonAkonadi/AddresseeLineEdit>
@@ -75,7 +75,6 @@ using MailTransport::TransportManager;
 #include <QFile>
 #include <QFileInfo>
 #include <QFormLayout>
-#include <QGridLayout>
 #include <QHostInfo>
 #include <QLabel>
 #include <QToolButton>
@@ -92,6 +91,7 @@ using MailTransport::TransportManager;
 #include <Akonadi/SpecialMailCollections>
 #include <QDialogButtonBox>
 #include <QStandardPaths>
+#include <QGroupBox>
 
 using namespace KPIM;
 using namespace MailTransport;
@@ -372,7 +372,39 @@ IdentityDialog::IdentityDialog(QWidget *parent)
     mPGPSigningKeyRequester->setWhatsThis(msg);
     label->setWhatsThis(msg);
 
-    formLayout->addRow(label, mPGPSigningKeyRequester);
+    auto vbox = new QVBoxLayout;
+    mPGPSameKey = new QCheckBox(i18n("Use same key for encryption and signing"));
+    vbox->addWidget(mPGPSigningKeyRequester);
+    vbox->addWidget(mPGPSameKey);
+    formLayout->addRow(label, vbox);
+
+    connect(mPGPSameKey, &QCheckBox::toggled, this, [=](bool checked) {
+        mPGPEncryptionKeyRequester->setVisible(!checked);
+        formLayout->labelForField(mPGPEncryptionKeyRequester)->setVisible(!checked);
+        const auto label = qobject_cast<QLabel *>(formLayout->labelForField(vbox));
+        if (checked) {
+            label->setText(i18n("OpenPGP key:"));
+            const auto key = mPGPSigningKeyRequester->currentKey();
+            if (!key.isBad()) {
+                mPGPEncryptionKeyRequester->setCurrentKey(key);
+            } else if (mPGPSigningKeyRequester->currentData() == QLatin1String("no-key")) {
+                mPGPEncryptionKeyRequester->setCurrentIndex(mPGPSigningKeyRequester->currentIndex());
+            }
+        } else {
+            label->setText(i18n("OpenPGP signing key:"));
+        }
+    });
+    connect(mPGPSigningKeyRequester, &KeySelectionCombo::currentKeyChanged, this, [&](const GpgME::Key &key) {
+        if (mPGPSameKey->isChecked()) {
+            mPGPEncryptionKeyRequester->setCurrentKey(key);
+        }
+    });
+    connect(mPGPSigningKeyRequester, &KeySelectionCombo::customItemSelected, this, [&](const QVariant& type) {
+        if (mPGPSameKey->isChecked() && type == QLatin1String("no-key")) {
+            mPGPEncryptionKeyRequester->setCurrentIndex(mPGPSigningKeyRequester->currentIndex());
+        }
+    });
+
 
     // "OpenPGP Encryption Key" requester and label:
     mPGPEncryptionKeyRequester = new KeySelectionCombo(KeySelectionCombo::EncryptionKey, GpgME::OpenPGP, mCryptographyTab);
@@ -386,8 +418,8 @@ IdentityDialog::IdentityDialog(QWidget *parent)
         "<p>You can find out more about keys at <a>https://www.gnupg.org</a></p></qt>");
     label = new QLabel(i18n("OpenPGP encryption key:"), mCryptographyTab);
     label->setBuddy(mPGPEncryptionKeyRequester);
-    mPGPEncryptionKeyRequester->setWhatsThis(msg);
     label->setWhatsThis(msg);
+    mPGPEncryptionKeyRequester->setWhatsThis(msg);
 
     formLayout->addRow(label, mPGPEncryptionKeyRequester);
 
@@ -441,11 +473,37 @@ IdentityDialog::IdentityDialog(QWidget *parent)
 
     formLayout->addRow(label, mPreferredCryptoMessageFormat);
 
-    mAutoSign = new QCheckBox(i18n("Automatically sign messages"));
-    formLayout->addWidget(mAutoSign);
+    mAutocrypt = new QGroupBox(i18n("Enable Autocrypt"));
+    mAutocrypt->setCheckable(true);
+    mAutocrypt->setChecked(true);
 
-    mAutoEncrypt = new QCheckBox(i18n("Automatically encrypt messages when possible"));
-    formLayout->addWidget(mAutoEncrypt);
+    label = new QLabel(i18n("Autocrypt:"), tab);
+    formLayout->addRow(label, mAutocrypt);
+
+    vlay = new QVBoxLayout(mAutocrypt);
+
+    mAutocryptPrefer = new QCheckBox(i18n("Let others know you prefer encryption"));
+    vlay->addWidget(mAutocryptPrefer);
+
+    mOverrideDefault = new QGroupBox(i18n("Overwrite global settings for security defaults"));
+    mOverrideDefault->setCheckable(true);
+    mOverrideDefault->setChecked(false);
+    label = new QLabel(i18n("Overwrite defaults:"), tab);
+    formLayout->addRow(label, mOverrideDefault);
+
+    vlay = new QVBoxLayout(mOverrideDefault);
+
+    mAutoSign = new QCheckBox(i18n("Sign messages"));
+    vlay->addWidget(mAutoSign);
+
+    mAutoEncrypt = new QCheckBox(i18n("Encrypt messages when possible"));
+    vlay->addWidget(mAutoEncrypt);
+
+    mWarnNotSign = new QCheckBox(i18n("Warn when trying to send unsigned messages"));
+    vlay->addWidget(mWarnNotSign);
+
+    mWarnNotEncrypt = new QCheckBox(i18n("Warn when trying to send unencrypted messages"));
+    vlay->addWidget(mWarnNotEncrypt);
 
     //
     // Tab Widget: Advanced
@@ -562,7 +620,7 @@ IdentityDialog::IdentityDialog(QWidget *parent)
     connect(mEditVCard, &QPushButton::clicked, this, &IdentityDialog::slotEditVcard);
     formLayout->addRow(mAttachMyVCard, mEditVCard);
 
-    mAutoCorrectionLanguage = new PimCommon::AutoCorrectionLanguage(tab);
+    mAutoCorrectionLanguage = new PimCommonAutoCorrection::AutoCorrectionLanguage(tab);
     label = new QLabel(i18n("Autocorrection language:"), tab);
     label->setBuddy(mAutoCorrectionLanguage);
     formLayout->addRow(label, mAutoCorrectionLanguage);
@@ -688,7 +746,7 @@ void IdentityDialog::slotAccepted()
         }
         if (!KEmailAddress::isValidSimpleAddress(alias)) {
             const QString errorMsg(KEmailAddress::simpleEmailAddressErrorMsg());
-            KMessageBox::sorry(this, errorMsg, i18n("Invalid Email Alias \"%1\"", alias));
+            KMessageBox::error(this, errorMsg, i18n("Invalid Email Alias \"%1\"", alias));
             return;
         }
     }
@@ -697,7 +755,7 @@ void IdentityDialog::slotAccepted()
     const QString email = mEmailEdit->text().trimmed();
     if (!KEmailAddress::isValidSimpleAddress(email)) {
         const QString errorMsg(KEmailAddress::simpleEmailAddressErrorMsg());
-        KMessageBox::sorry(this, errorMsg, i18n("Invalid Email Address"));
+        KMessageBox::error(this, errorMsg, i18n("Invalid Email Address"));
         return;
     }
 
@@ -829,12 +887,27 @@ void IdentityDialog::setIdentity(KIdentityManagement::Identity &ident)
     // "Cryptography" tab:
     mPGPSigningKeyRequester->setDefaultKey(QLatin1String(ident.pgpSigningKey()));
     mPGPEncryptionKeyRequester->setDefaultKey(QLatin1String(ident.pgpEncryptionKey()));
+
+    mPGPSameKey->setChecked(ident.pgpSigningKey() == ident.pgpEncryptionKey());
+
     mSMIMESigningKeyRequester->setDefaultKey(QLatin1String(ident.smimeSigningKey()));
     mSMIMEEncryptionKeyRequester->setDefaultKey(QLatin1String(ident.smimeEncryptionKey()));
 
     mPreferredCryptoMessageFormat->setCurrentIndex(format2cb(Kleo::stringToCryptoMessageFormat(ident.preferredCryptoMessageFormat())));
-    mAutoSign->setChecked(ident.pgpAutoSign());
-    mAutoEncrypt->setChecked(ident.pgpAutoEncrypt());
+    mAutocrypt->setChecked(ident.autocryptEnabled());
+    mAutocryptPrefer->setChecked(ident.autocryptPrefer());
+    mOverrideDefault->setChecked(ident.encryptionOverride());
+    if (!ident.encryptionOverride()) {
+        mAutoSign->setChecked(ident.pgpAutoSign());
+        mAutoEncrypt->setChecked(ident.pgpAutoEncrypt());
+        mWarnNotSign->setChecked(ident.warnNotSign());
+        mWarnNotEncrypt->setChecked(ident.warnNotEncrypt());
+    } else {
+        mAutoEncrypt->setChecked(MessageComposer::MessageComposerSettings::self()->cryptoAutoEncrypt());
+        mAutoSign->setChecked(MessageComposer::MessageComposerSettings::self()->cryptoAutoSign());
+        mWarnNotEncrypt->setChecked(MessageComposer::MessageComposerSettings::self()->cryptoWarningUnencrypted());
+        mWarnNotSign->setChecked(MessageComposer::MessageComposerSettings::self()->cryptoWarningUnsigned());
+    }
 
     // "Advanced" tab:
     mReplyToEdit->setText(ident.replyToAddr());
@@ -953,8 +1026,13 @@ void IdentityDialog::updateIdentity(KIdentityManagement::Identity &ident)
     ident.setSMIMESigningKey(mSMIMESigningKeyRequester->currentKey().primaryFingerprint());
     ident.setSMIMEEncryptionKey(mSMIMEEncryptionKeyRequester->currentKey().primaryFingerprint());
     ident.setPreferredCryptoMessageFormat(QLatin1String(Kleo::cryptoMessageFormatToString(cb2format(mPreferredCryptoMessageFormat->currentIndex()))));
+    ident.setAutocryptEnabled(mAutocrypt->isChecked());
+    ident.setAutocryptPrefer(mAutocryptPrefer->isChecked());
+    ident.setEncryptionOverride(mOverrideDefault->isChecked());
     ident.setPgpAutoSign(mAutoSign->isChecked());
     ident.setPgpAutoEncrypt(mAutoEncrypt->isChecked());
+    ident.setWarnNotEncrypt(mWarnNotEncrypt->isChecked());
+    ident.setWarnNotEncrypt(mWarnNotEncrypt->isChecked());
     // "Advanced" tab:
     ident.setReplyToAddr(mReplyToEdit->text());
     ident.setBcc(mBccEdit->text());
